# -*- coding: utf-8 -*-


from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('django_nyt', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Settings',
            fields=[
                ('id', models.AutoField(auto_created=True, verbose_name='ID', serialize=False, primary_key=True)),
                ('user', models.ForeignKey(to_field='id', to=settings.AUTH_USER_MODEL)),
                ('interval', models.SmallIntegerField(choices=[(0, 'instantly'), (1380, 'daily'), (9660, 'weekly')], default=0, verbose_name='interval')),
            ],
            options={
                'verbose_name': 'settings',
                'db_table': 'nyt_settings',
                'verbose_name_plural': 'settings',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.AutoField(auto_created=True, verbose_name='ID', serialize=False, primary_key=True)),
                ('message', models.TextField()),
                ('url', models.CharField(verbose_name='link for notification', null=True, max_length=200, blank=True)),
                ('is_viewed', models.BooleanField(default=False)),
                ('is_emailed', models.BooleanField(default=False)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('occurrences', models.PositiveIntegerField(help_text='If the same notification was fired multiple times with no intermediate notifications', default=1, verbose_name='occurrences')),
            ],
            options={
                'verbose_name': 'notification',
                'db_table': 'nyt_notification',
                'ordering': ('-id',),
                'verbose_name_plural': 'notifications',
            },
            bases=(models.Model,),
        ),
    ]



from . import _disable_notifications

from django.db.models import Model
from django.utils.translation import ugettext as _
from . import models


def notify(message, key, target_object=None, url=None, filter_exclude={}):
    """
    Notify subscribing users of a new event. Key can be any kind of string,
    just make sure to reuse it where applicable! Object_id is some identifier
    of an object, for instance if a user subscribes to a specific comment thread,
    you could write:
    
    notify("there was a response to your comment", "comment_response",
           target_object=PostersObject,
           url=reverse('comments:view', args=(PostersObject.id,)))
    
    The below example notifies everyone subscribing to the "new_comments" key
    with the message "New comment posted".
    
    notify("New comment posted", "new_comments")
    
    filter_exclude: a dictionary to exclude special elements of subscriptions
    in the queryset, for instance filter_exclude={''}
    
    """

    if _disable_notifications:
        return 0
    
    if target_object:
        if not isinstance(target_object, Model):
            raise TypeError(_("You supplied a target_object that's not an instance of a django Model."))
        object_id = target_object.id
    else:
        object_id = None
        
    objects = models.Notification.create_notifications(
        key,
        object_id=object_id,
        message=message,
        url=url,
        filter_exclude=filter_exclude,
    )
    return len(objects)



# -*- coding: utf-8 -*-


from django.db import models, migrations
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_nyt', '0003_subscription'),
    ]

    operations = [
        migrations.AddField(
            model_name='notification',
            name='subscription',
            field=models.ForeignKey(to_field='id', to='django_nyt.Subscription', blank=True, on_delete=django.db.models.deletion.SET_NULL, null=True),
            preserve_default=True,
        ),
    ]



from django.conf import settings as django_settings
from django import VERSION as DJANGO_VERSION
from django.utils.translation import ugettext_lazy as _


DB_TABLE_PREFIX = 'nyt'

# You need to switch this setting on, otherwise nothing will happen :)
ENABLED = getattr(django_settings, 'NYT_ENABLED', True)

# Enable django-admin registration
ENABLE_ADMIN = getattr(django_settings, 'NYT_ENABLE_ADMIN', False)

# Email notifications won't get sent unless you run
# python manage.py notifymail
SEND_EMAILS = getattr(django_settings, 'NYT_SEND_EMAILS', True)

EMAIL_SUBJECT = getattr(django_settings, 
    'NYT_EMAIL_SUBJECT', _("You have new notifications")) 

EMAIL_SENDER = getattr(django_settings, 
    'NYT_EMAIL_SENDER', "notifications@example.com")

# Seconds to sleep between each database poll
# (leave high unless you really want to send extremely real time
# notifications)
NYT_SLEEP_TIME = 120

# You can always make up more numbers... they simply identify which notifications
# to send when invoking the script, and the number indicates how many hours
# to minimum pass between each notification.
INSTANTLY = 0
DAILY = (24 - 1) * 60 # Subtract 1, because the job finishes less than 24h before the next...
WEEKLY = 7 * (24 - 1) * 60

# List of intervals available. In minutes
INTERVALS = getattr(django_settings, 'NYT_INTERVALS',
    [(INSTANTLY, _('instantly')),
     (DAILY, _('daily')),
     (WEEKLY, _('weekly'))]
)

INTERVALS_DEFAULT = INSTANTLY

# Django 1.5+
if DJANGO_VERSION >= (1,5):
    USER_MODEL = getattr(django_settings, 'AUTH_USER_MODEL', 'auth.User')
else:
    USER_MODEL = 'auth.User'

NYT_LOG = getattr(django_settings, 'NYT_LOG', '/tmp/nyt_daemon.log')
NYT_PID = getattr(django_settings, 'NYT_PID', '/tmp/nyt_daemon.pid')

####################
# PLANNED SETTINGS #
####################

# Minimum logging and digital garbage! Don't save too much crap!

# After how many days should viewed notifications be deleted?
AUTO_DELETE = getattr(django_settings, 'NYT_AUTO_DELETE', 120)

# After how many days should all types of notifications be deleted?
AUTO_DELETE_ALL = getattr(django_settings, 'NYT_AUTO_DELETE_ALL', 120)





# -*- coding: utf-8 -*-


from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '__first__'),
    ]

    operations = [
        migrations.CreateModel(
            name='NotificationType',
            fields=[
                ('key', models.CharField(verbose_name='unique key', serialize=False, max_length=128, primary_key=True, unique=True)),
                ('label', models.CharField(verbose_name='verbose name', null=True, max_length=128, blank=True)),
                ('content_type', models.ForeignKey(to_field='id', to='contenttypes.ContentType', blank=True, null=True)),
            ],
            options={
                'verbose_name': 'type',
                'db_table': 'nyt_notificationtype',
                'verbose_name_plural': 'types',
            },
            bases=(models.Model,),
        ),
    ]



from django.test import TestCase

from django_nyt import notify, models

try:
    from django.contrib.auth import get_user_model
    User = get_user_model()
except ImportError:
    from django.contrib.auth.models import User


class NotifyTest(TestCase):
    
    def test_simple(self):
        
        TEST_KEY = 'test_key'
        TEST_USER = User.objects.create_user(
            'lalala'
        )
        
        TEST_NOTIFICATION_TYPE = models.NotificationType.objects.create(key=TEST_KEY)
        TEST_SETTINGS = models.Settings.objects.create(
            user=TEST_USER,
        )
        models.Subscription.objects.create(
            settings=TEST_SETTINGS,
            notification_type=TEST_NOTIFICATION_TYPE,
        )
        notify("Test Is a Test", TEST_KEY)
        
        self.assertEqual(models.Notification.objects.all().count(), 1)


from django import VERSION as DJANGO_VERSION


def get_user_model():
    
    if DJANGO_VERSION >= (1, 5):
        from django.contrib.auth import get_user_model as gum
        return gum()
    else:
        from django.contrib.auth.models import User
        return User



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'NotificationType'
        db.create_table('nyt_notificationtype', (
            ('key', self.gf('django.db.models.fields.CharField')(unique=True, max_length=128, primary_key=True)),
            ('label', self.gf('django.db.models.fields.CharField')(max_length=128, null=True, blank=True)),
            ('content_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'], null=True, blank=True)),
        ))
        db.send_create_signal('django_nyt', ['NotificationType'])

        # Adding model 'Settings'
        db.create_table('nyt_settings', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('interval', self.gf('django.db.models.fields.SmallIntegerField')(default=0)),
        ))
        db.send_create_signal('django_nyt', ['Settings'])

        # Adding model 'Subscription'
        db.create_table('nyt_subscription', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('settings', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['django_nyt.Settings'])),
            ('notification_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['django_nyt.NotificationType'])),
            ('object_id', self.gf('django.db.models.fields.CharField')(max_length=64, null=True, blank=True)),
            ('send_emails', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('latest', self.gf('django.db.models.fields.related.ForeignKey')(blank=True, related_name='latest_for', null=True, to=orm['django_nyt.Notification'])),
        ))
        db.send_create_signal('django_nyt', ['Subscription'])

        # Adding model 'Notification'
        db.create_table('nyt_notification', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('subscription', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['django_nyt.Subscription'], null=True, on_delete=models.SET_NULL, blank=True)),
            ('message', self.gf('django.db.models.fields.TextField')()),
            ('url', self.gf('django.db.models.fields.CharField')(max_length=200, null=True, blank=True)),
            ('is_viewed', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('is_emailed', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('created', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('occurrences', self.gf('django.db.models.fields.PositiveIntegerField')(default=1)),
        ))
        db.send_create_signal('django_nyt', ['Notification'])


    def backwards(self, orm):
        # Deleting model 'NotificationType'
        db.delete_table('nyt_notificationtype')

        # Deleting model 'Settings'
        db.delete_table('nyt_settings')

        # Deleting model 'Subscription'
        db.delete_table('nyt_subscription')

        # Deleting model 'Notification'
        db.delete_table('nyt_notification')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'django_nyt.notification': {
            'Meta': {'ordering': "('-id',)", 'object_name': 'Notification', 'db_table': "'nyt_notification'"},
            'created': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_emailed': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_viewed': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'message': ('django.db.models.fields.TextField', [], {}),
            'occurrences': ('django.db.models.fields.PositiveIntegerField', [], {'default': '1'}),
            'subscription': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['django_nyt.Subscription']", 'null': 'True', 'on_delete': 'models.SET_NULL', 'blank': 'True'}),
            'url': ('django.db.models.fields.CharField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'})
        },
        'django_nyt.notificationtype': {
            'Meta': {'object_name': 'NotificationType', 'db_table': "'nyt_notificationtype'"},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']", 'null': 'True', 'blank': 'True'}),
            'key': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '128', 'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '128', 'null': 'True', 'blank': 'True'})
        },
        'django_nyt.settings': {
            'Meta': {'object_name': 'Settings', 'db_table': "'nyt_settings'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'interval': ('django.db.models.fields.SmallIntegerField', [], {'default': '0'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'django_nyt.subscription': {
            'Meta': {'object_name': 'Subscription', 'db_table': "'nyt_subscription'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'latest': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'latest_for'", 'null': 'True', 'to': "orm['django_nyt.Notification']"}),
            'notification_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['django_nyt.NotificationType']"}),
            'object_id': ('django.db.models.fields.CharField', [], {'max_length': '64', 'null': 'True', 'blank': 'True'}),
            'send_emails': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'settings': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['django_nyt.Settings']"})
        }
    }

    complete_apps = ['django_nyt']


# -*- coding: utf-8 -*-
_disable_notifications = False

VERSION = "0.9.1"

def notify(*args, **kwargs):
    """
    DEPRECATED - please access django_nyt.utils.notify
    """
    from django_nyt.utils import notify
    return notify(*args, **kwargs)



# -*- coding: utf-8 -*-
from django.conf.urls import patterns, url

urlpatterns = patterns('',
    url('^json/get/$', 'django_nyt.views.get_notifications', name='json_get'),   
    url('^json/get/(?P<latest_id>\d+)/$', 'django_nyt.views.get_notifications', name='json_get'),   
    url('^json/mark-read/$', 'django_nyt.views.mark_read', name='json_mark_read_base'),   
    url('^json/mark-read/(\d+)/$', 'django_nyt.views.mark_read', name='json_mark_read'),   
    url('^json/mark-read/(?P<id_lte>\d+)/(?P<id_gte>\d+)/$', 'django_nyt.views.mark_read', name='json_mark_read'),   
    url('^goto/(?P<notification_id>\d+)/$', 'django_nyt.views.goto', name='goto'),   
    url('^goto/$', 'django_nyt.views.goto', name='goto_base'),   
)

def get_pattern(app_name="nyt", namespace="nyt"):
    """Every url resolution takes place as "nyt:view_name".
       https://docs.djangoproject.com/en/dev/topics/http/urls/#topics-http-reversing-url-namespaces
    """
    return urlpatterns, app_name, namespace


import os
import sys
import time
import smtplib
import logging
from datetime import datetime
from optparse import make_option

from django.contrib.sites.models import Site
from django.core import mail
from django.core.management.base import BaseCommand
from django.template.loader import render_to_string
from django.utils.translation import ugettext as _, activate, deactivate
from django.conf import settings

from django_nyt import models
from django_nyt import settings as nyt_settings



class Command(BaseCommand):
    can_import_settings = True
    help = 'Sends notification emails to subscribed users taking into account the subscription interval' #@ReservedAssignment
    option_list = BaseCommand.option_list + (
        make_option('--daemon','-d',
        action='store_true',
        dest='daemon',
        default=False,
        help='Go to daemon mode and exit'),
    )

    def _send_user_notifications(self, context, connection):
        subject = _(nyt_settings.EMAIL_SUBJECT)
        message = render_to_string(
            'emails/notification_email_message.txt',
            context
        )
        email = mail.EmailMessage(
            subject, message, nyt_settings.EMAIL_SENDER,
            [context['user'].email], connection=connection
        )
        self.logger.info("Sending to: %s" % context['user'].email)
        email.send(fail_silently=False)

    def handle(self, *args, **options):
        # activate the language
        activate(settings.LANGUAGE_CODE)

        daemon = options['daemon']

        self.logger = logging.getLogger('django_nyt')

        if not self.logger.handlers:
            if daemon:
                handler = logging.FileHandler(filename=nyt_settings.NYT_LOG)
            else:
                handler = logging.StreamHandler(self.stdout)
            self.logger.addHandler(handler)
            self.logger.setLevel(logging.INFO)

        self.logger.info("Starting django_nyt e-mail dispatcher")

        if not nyt_settings.SEND_EMAILS:
            print("E-mails disabled - quitting.")
            sys.exit()

        # Run as daemon, ie. fork the process
        if daemon:
            self.logger.info("Daemon mode enabled, forking")
            try:
                fpid = os.fork()
                if fpid > 0:
                # Running as daemon now. PID is fpid
                    self.logger.info("PID: %s" % str(fpid))
                    pid_file = file(nyt_settings.NYT_PID, "w")
                    pid_file.write(str(fpid))
                    pid_file.close()
                    sys.exit(0)
            except OSError as e:
                sys.stderr.write("fork failed: %d (%s)\n" % (e.errno, e.strerror))
                sys.exit(1)

        try:
            self.send_loop()
        except KeyboardInterrupt:
            print("\nQuitting...")

        # deactivate the language
        deactivate()

    def send_loop(self):

        # This could be /improved by looking up the last notified person
        last_sent = None
        context = {'user': None,
                   'notifications': None,
                   'digest': None,
                   'site': Site.objects.get_current()}

        #create a connection to smtp server for reuse
        try:
            connection = mail.get_connection()
        except:
            self.logger.error("Could get a mail connection")
            raise

        while True:
            try:
                connection.open()
            except:
                self.logger.error("Could not use e-mail connection")
                raise
            started_sending_at = datetime.now()

            self.logger.info("Starting send loop at %s" % str(started_sending_at))

            if last_sent:
                user_settings = models.Settings.objects.filter(
                    interval__lte=((started_sending_at-last_sent).seconds // 60) // 60
                ).order_by('user')
            else:
                user_settings = models.Settings.objects.all().order_by('user')

            for setting in user_settings:
                context['user'] = setting.user
                context['notifications']= []
                #get the index of the tuple corresponding to the interval and get the string name
                context['digest'] = nyt_settings.INTERVALS[[y[0] for y in nyt_settings.INTERVALS].index(setting.interval)][1]
                for subscription in setting.subscription_set.filter(
                    send_emails=True,
                    latest__is_emailed=False
                ):
                    context['notifications'].append(subscription.latest)
                if len(context['notifications']) > 0:
                    try:
                        self._send_user_notifications(context, connection)
                        for n in context['notifications']:
                            n.is_emailed=True
                            n.save()
                    except smtplib.SMTPException:
                        # TODO: Only quit on certain errors, retry on others.
                        self.logger.error("You have an error with your SMTP server connection, quitting.")
                        raise

            connection.close()
            last_sent = datetime.now()
            elapsed_seconds = (last_sent - started_sending_at).seconds
            time.sleep(
                max(
                    (min(nyt_settings.INTERVALS)[0] - elapsed_seconds) * 60,
                    nyt_settings.NOTIFY_SLEEP_TIME,
                    0
                )
            )



# -*- coding: utf-8 -*-
from django.db import models
from django.db.models import Q
from django.utils.translation import ugettext_lazy as _

from django.contrib.contenttypes.models import ContentType

from django_nyt import settings


class NotificationType(models.Model):
    """
    Notification types are added on-the-fly by the
    applications adding new notifications
    """
    key = models.CharField(
        max_length=128, 
        primary_key=True, 
        verbose_name=_('unique key'),
        unique=True
    )
    label = models.CharField(
        max_length=128, 
        verbose_name=_('verbose name'),
        blank=True, 
        null=True
    )
    content_type = models.ForeignKey(ContentType, blank=True, null=True)
    
    def __unicode__(self):
        return self.key
    
    class Meta:
        db_table = settings.DB_TABLE_PREFIX + '_notificationtype'
        verbose_name = _('type')
        verbose_name_plural = _('types')


class Settings(models.Model):
    """
    Reusable settings object for a subscription
    """
    
    user = models.ForeignKey(
        settings.USER_MODEL
    )
    interval = models.SmallIntegerField(
        choices=settings.INTERVALS, 
        verbose_name=_('interval'),
        default=settings.INTERVALS_DEFAULT
    )
    
    def __unicode__(self):
        obj_name = _("Settings for %s") % self.user.username
        return str(obj_name)
    
    class Meta:
        db_table = settings.DB_TABLE_PREFIX + '_settings'
        verbose_name = _('settings')
        verbose_name_plural = _('settings')


class Subscription(models.Model):
    
    settings = models.ForeignKey(Settings)
    notification_type = models.ForeignKey(NotificationType)
    object_id = models.CharField(
        max_length=64, 
        null=True, 
        blank=True, 
        help_text=_('Leave this blank to subscribe to any kind of object')
    )
    send_emails = models.BooleanField(default=True)
    latest = models.ForeignKey('Notification', 
        null=True, 
        blank=True, 
        related_name='latest_for'
    )
    
    def __unicode__(self):
        obj_name = _("Subscription for: %s") % str(self.settings.user.username)
        return str(obj_name)

    class Meta:
        db_table = settings.DB_TABLE_PREFIX + '_subscription'
        verbose_name = _('subscription')
        verbose_name_plural = _('subscriptions')


class Notification(models.Model):
    
    subscription = models.ForeignKey(
        Subscription, 
        null=True, 
        blank=True, 
        on_delete=models.SET_NULL
    )
    message = models.TextField()
    url = models.CharField(
        verbose_name=_('link for notification'),
        blank=True, 
        null=True, 
        max_length=200
    )
    is_viewed = models.BooleanField(default=False)
    is_emailed = models.BooleanField(default=False)
    created = models.DateTimeField(auto_now_add=True)
    occurrences = models.PositiveIntegerField(
        default=1, 
        verbose_name=_('occurrences'),
        help_text=_(
            'If the same notification was fired multiple '
             'times with no intermediate notifications'
         )
    )
    
    @classmethod
    def create_notifications(cls, key, **kwargs):
        """Creates notifications directly in database -- do not call directly, use django_nyt.notify(...)"""
        if not key or not isinstance(key, str):
            raise KeyError('No notification key (string) specified.')
        
        object_id = kwargs.pop('object_id', None)
        filter_exclude = kwargs.pop('filter_exclude', {})
        
        objects_created = []
        subscriptions = Subscription.objects.filter(
            Q(notification_type__key=key) | 
            Q(notification_type__key=None),
            **filter_exclude
        )
        if object_id:
            subscriptions = subscriptions.filter(
                Q(object_id=object_id) |
                Q(object_id=None)
            )

        subscriptions = subscriptions.prefetch_related('latest', 'settings')
        subscriptions = subscriptions.order_by('settings__user')
        prev_user = None
        
        for subscription in subscriptions:
            # Don't alert the same user several times even though overlapping
            # subscriptions occur.
            if subscription.settings.user == prev_user:
                continue
            
            # Check if it's the same as the previous message
            latest = subscription.latest
            if latest and (latest.message == kwargs.get('message', None) and 
                latest.url == kwargs.get('url', None) and
                latest.is_viewed == False):
                # Both message and URL are the same, and it hasn't been viewed
                # so just increment occurrence count.
                latest.occurrences = latest.occurrences + 1
                latest.is_emailed = False
                latest.save()
            else:
                # Insert a new notification
                new_obj = cls.objects.create(subscription=subscription, **kwargs)
                objects_created.append(
                   new_obj
                )
                subscription.latest = new_obj
                subscription.save()
            prev_user = subscription.settings.user
        
        return objects_created
    
    def __unicode__(self):
        return "%s: %s" % (str(self.subscription.settings.user), self.message)

    class Meta:
        db_table = settings.DB_TABLE_PREFIX + '_notification'
        verbose_name = _('notification')
        verbose_name_plural = _('notifications')
        ordering = ('-id',)



# -*- coding: utf-8 -*-


import os
from django_nyt import VERSION
from setuptools import setup, find_packages

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...


def get_path(fname):
    return os.path.join(os.path.dirname(os.path.abspath(__file__)), fname)


def read(fname):
    return open(get_path(fname)).read()

packages = find_packages()

try:
    import pypandoc
    long_description = pypandoc.convert(get_path('README.md'), 'rst')
    long_description = long_description.split(
        '<!---Illegal PyPi RST data -->')[0]
    f = open(get_path('README.rst'), 'w')
    f.write(long_description)
    f.close()
    print("Successfully converted README.md to README.rst")
except (IOError, ImportError):
    # No long description... but nevermind, it's only for PyPi uploads.
    long_description = ""

setup(
    name="django-nyt",
    version=VERSION,
    author="Benjamin Bach",
    author_email="benjamin@overtag.dk",
    url="https://github.com/benjaoming/django-nyt",
    description="A pluggable notification system written for the Django framework.",
    license="Apache License 2.0",
    keywords="django notification system",
    packages=find_packages(exclude=["testproject", "testproject.*"]),
    # long_description=long_description,
    zip_safe=False,
    install_requires=read('requirements.txt').split("\n"),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
    ],
    include_package_data=True,
)



# -*- coding: utf-8 -*-


from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_nyt', '0002_notification_settings'),
    ]

    operations = [
        migrations.CreateModel(
            name='Subscription',
            fields=[
                ('id', models.AutoField(auto_created=True, verbose_name='ID', serialize=False, primary_key=True)),
                ('settings', models.ForeignKey(to_field='id', to='django_nyt.Settings')),
                ('notification_type', models.ForeignKey(to_field='key', to='django_nyt.NotificationType')),
                ('object_id', models.CharField(help_text='Leave this blank to subscribe to any kind of object', null=True, max_length=64, blank=True)),
                ('send_emails', models.BooleanField(default=True)),
                ('latest', models.ForeignKey(to_field='id', to='django_nyt.Notification', blank=True, null=True)),
            ],
            options={
                'verbose_name': 'subscription',
                'db_table': 'nyt_subscription',
                'verbose_name_plural': 'subscriptions',
            },
            bases=(models.Model,),
        ),
    ]
